/*
 * $RCSfile: TIFFYCbCrColorConverter.java,v $
 *
 * 
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met: 
 * 
 * - Redistribution of source code must retain the above copyright 
 *   notice, this  list of conditions and the following disclaimer.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in 
 *   the documentation and/or other materials provided with the
 *   distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any 
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND 
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL 
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR 
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES. 
 * 
 * You acknowledge that this software is not designed or intended for 
 * use in the design, construction, operation or maintenance of any 
 * nuclear facility. 
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/11 22:10:37 $
 * $State: Exp $
 */
package com.sun.media.imageioimpl.plugins.tiff;

import com.sun.media.imageio.plugins.tiff.BaselineTIFFTagSet;
import com.sun.media.imageio.plugins.tiff.TIFFColorConverter;
import com.sun.media.imageio.plugins.tiff.TIFFField;

public final class TIFFYCbCrColorConverter extends TIFFColorConverter {
	private float _lumaRed = 0.299f;
	private float _lumaGreen = 0.587f;
	private float _lumaBlue = 0.114f;

	private float _referenceBlackY = 0.0f;
	private float _referenceWhiteY = 255.0f;

	private float _referenceBlackCb = 128.0f;
	private float _referenceWhiteCb = 255.0f;

	private float _referenceBlackCr = 128.0f;
	private float _referenceWhiteCr = 255.0f;

	private float _codingRangeY = 255.0f;
	private float _codingRangeCbCr = 127.0f;

	public TIFFYCbCrColorConverter(TIFFImageMetadata metadata) {
		TIFFImageMetadata tmetadata = metadata;

		TIFFField f =
				tmetadata.getTIFFField(BaselineTIFFTagSet.TAG_Y_CB_CR_COEFFICIENTS);
		if (f != null && f.getCount() == 3) {
			_lumaRed = f.getAsFloat(0);
			_lumaGreen = f.getAsFloat(1);
			_lumaBlue = f.getAsFloat(2);
		}

		f =
				tmetadata.getTIFFField(BaselineTIFFTagSet.TAG_REFERENCE_BLACK_WHITE);
		if (f != null && f.getCount() == 6) {
			_referenceBlackY = f.getAsFloat(0);
			_referenceWhiteY = f.getAsFloat(1);
			_referenceBlackCb = f.getAsFloat(2);
			_referenceWhiteCb = f.getAsFloat(3);
			_referenceBlackCr = f.getAsFloat(4);
			_referenceWhiteCr = f.getAsFloat(5);
		}
	}

	/*
      The full range component value is converted from the code by:

      FullRangeValue = (code - ReferenceBlack) * CodingRange
                / (ReferenceWhite - ReferenceBlack);

      The code is converted from the full-range component value by:

      code = (FullRangeValue * (ReferenceWhite - ReferenceBlack)
                / CodingRange) + ReferenceBlack;
	 */
	@Override
	public void fromRGB(float r, float g, float b, float[] result) {
		// Convert RGB to full-range YCbCr.
		float y = (_lumaRed * r + _lumaGreen * g + _lumaBlue * b);
		float cb = (b - y) / (2 - 2 * _lumaBlue);
		float cr = (r - y) / (2 - 2 * _lumaRed);

		// Convert full-range YCbCr to code.
		result[0] = y * (_referenceWhiteY - _referenceBlackY) / _codingRangeY
				+ _referenceBlackY;
		result[1] = cb * (_referenceWhiteCb - _referenceBlackCb) / _codingRangeCbCr
				+ _referenceBlackCb;
		result[2] = cr * (_referenceWhiteCr - _referenceBlackCr) / _codingRangeCbCr
				+ _referenceBlackCr;
	}

	@Override
	public void toRGB(float x0, float x1, float x2, float[] rgb) {
		// Convert YCbCr code to full-range YCbCr.
		float y = (x0 - _referenceBlackY) * _codingRangeY
				/ (_referenceWhiteY - _referenceBlackY);
		float cb = (x1 - _referenceBlackCb) * _codingRangeCbCr
				/ (_referenceWhiteCb - _referenceBlackCb);
		float cr = (x2 - _referenceBlackCr) * _codingRangeCbCr
				/ (_referenceWhiteCr - _referenceBlackCr);

		// Convert YCbCr to RGB.
		rgb[0] = cr * (2 - 2 * _lumaRed) + y;
		rgb[2] = cb * (2 - 2 * _lumaBlue) + y;
		rgb[1] = (y - _lumaBlue * rgb[2] - _lumaRed * rgb[0]) / _lumaGreen;
	}
}
