package fr.greyc.pantheon.ariane.graph;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

public final class OngoingConnection {
	private final Point _connectionStartPoint; // Stored inside graph coordinates
	private Point _connectionEndPoint;

	public OngoingConnection( Graph graph, int x, int y ) {
		Graphics graphicContext = graph.getGraphics();
		_connectionEndPoint = screenToGraph(graph, x, y);
		_connectionStartPoint = screenToGraph(graph, x, y);
		drawConnectionThread(graphicContext, _connectionStartPoint, _connectionEndPoint);
		graphicContext.dispose();
	}

	public void paint( Graphics graphicContext ) {
		drawConnectionThread(graphicContext, _connectionStartPoint, _connectionEndPoint);
	}

	public void drawConnectionThread( Graph graph, Point coordinate ) {
		Graphics graphicContext = graph.getGraphics();
		drawConnectionThread(graphicContext, _connectionStartPoint, _connectionEndPoint);
		_connectionEndPoint = screenToGraph(graph, coordinate.x, coordinate.y);
		drawConnectionThread(graphicContext, _connectionStartPoint, _connectionEndPoint);
		graphicContext.dispose();
		Rectangle rectangle = new Rectangle(_connectionEndPoint.x, _connectionEndPoint.y, 1, 1);
		graph.scrollRectToVisible(rectangle);
	}

	private void drawConnectionThread( Graphics graphics, Point p1, Point p2 ) {
		Color savedColor = graphics.getColor();
		graphics.setColor(Color.RED);
		graphics.setXORMode(Color.GREEN); // -> links appear in blue
		drawLine(graphics, p1.x, p1.y, p2.x, p2.y);
		graphics.setColor(savedColor);
		graphics.setPaintMode();
	}

	private Point screenToGraph( Graph graph, int x, int y ) {
		Point p1 = graph.getParent().getLocationOnScreen();
		Point p2 = graph.getLocation();
		return new Point(x - p1.x - p2.x, y - p1.y - p2.y);
	}

	// Bresenham's line drawing algorithm, taken from:
	// http://graphics.lcs.mit.edu/~mcmillan/comp136/Lecture6/Lines.html
	// The line-drawing routine in the AWT is actually buggy.  When I
	// draw lines using graphics contexts from different sources (e.g.
	// passed in to the repaint() routine, or gotten with getGraphics()),
	// I get different lines, which leaves artifacts when redrawing
	// rubber-bands.
	private void drawLine( Graphics graphicContext, int x1, int y1, int x2, int y2 ) {
		int dy = y2 - y1;
		int dx = x2 - x1;
		int stepx;
		int stepy;

		if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
		if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
		dy <<= 1; // dy is now 2*dy
		dx <<= 1; // dx is now 2*dx

		int x = x1;
		int y = y1;
		graphicContext.drawLine(x, y, x, y);
		if (dx > dy) {
			int fraction = dy - (dx >> 1);    // same as 2*dy - dx
			while (x != x2) {
				if (fraction >= 0) {
					y += stepy;
					fraction -= dx;           // same as fraction -= 2*dx
				}
				x += stepx;
				fraction += dy;               // same as fraction -= 2*dy
				graphicContext.drawLine(x, y, x, y);
			}
		} else {
			int fraction = dx - (dy >> 1);
			while (y != y2) {
				if (fraction >= 0) {
					x += stepx;
					fraction -= dy;
				}
				y += stepy;
				fraction += dx;
				graphicContext.drawLine(x, y, x, y);
			}
		}
	}
}
