/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph;

import java.io.File;
import java.util.List;
import java.util.Map;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.graph.item.DataFile;
import fr.greyc.pantheon.ariane.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.graph.item.LoaderItem;
import fr.greyc.pantheon.ariane.graph.item.SaverItem;

/**
 * A <code>ThreadExecution</code> controls the execution of a graph
 * for a whole directory.
 *
 * @author Régis CLOUARD - Dec 15, 2011
 */
public final class ThreadDirectoryExecution extends Thread {
	/** The list of items to be executed. */
	private GraphItem[] _currentSelectedItems;
	private final LoaderItem _loader;
	private final File[] _files;
	private final Map<SaverItem, File> _saverDirectories;
	private final File _loaderFile;

	/**
	 * Creates a graph execution thread.
	 * @param selectedItems the list of item to be executed.
	 */
	public ThreadDirectoryExecution( List<GraphItem> selectedItems, LoaderItem loader, File loaderFile, File[] files, Map<SaverItem, File> saverDirectories) { // NOPMD by regis on 7/23/14 7:41 PM
		// We use a copy of the selectItems since user can modify the list
		// while the program is executing it.
		_currentSelectedItems = selectedItems.toArray(new GraphItem[selectedItems.size()]);
		_loader = loader;
		_files = files;
		_loaderFile = loaderFile;
		_saverDirectories = saverDirectories;
	}

	/**
	 * Runs the thread.
	 */
	@Override
	public void run( ) {
		Ariane.getWorkspace().setExecution(true);
		for ( File file : _files ) {
			String filename = file.getName();
			int suffixPosition = filename.lastIndexOf('.');
			String basename = filename;
			if (suffixPosition > 0) {
				basename = filename.substring(0, suffixPosition);
			}
			_loader.setBinary(file.getAbsolutePath());
			_loader.setTemporaryNickName(DataFile.getFileNameWithoutExtension(file.getName()));

			for ( SaverItem item : _saverDirectories.keySet()) {
				filename = _saverDirectories.get(item).getName();
				String suffix = "";
				suffixPosition = filename.lastIndexOf('.');
				if (suffixPosition > 0) {
					suffix = filename.substring(suffixPosition);
				}
				item.setBinary(_saverDirectories.get(item).getParent() + File.separator + basename + suffix);
			}
			_loader.resetAll();
			
			for ( GraphItem item: _currentSelectedItems ) {
				item.execute();
			}
		}
		_currentSelectedItems = null;
		Ariane.getWorkspace().setExecution(false);
		
		// Restore old file name to the loader and the savers (modified the execution).
		_loader.setBinary(_loaderFile.getAbsolutePath());
		_loader.setTemporaryNickName(DataFile.getFileNameWithoutExtension(_loaderFile.getAbsolutePath()));
		for ( SaverItem item : _saverDirectories.keySet()) {
			item.setBinary(_saverDirectories.get(item).getAbsolutePath());
		}
	}
}