/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.command;

import javax.swing.JComponent;
import javax.swing.undo.AbstractUndoableEdit;

import fr.greyc.pantheon.ariane.graph.item.GraphItem;

/**
 * This class is a part of the history manager.
 * It stores an add item action.
 * 
 * @author Régis CLOUARD - Apr 14, 2011
 */
public final class ChangeBinaryEdit extends AbstractUndoableEdit {
	private final GraphItem _graphItem;
	private final JComponent _textfield;
	/** The old value. */
	private final String _newValue;
	/** The old value. */
	private final String _oldValue;

	/**
	 * Creates a new edit.
	 * @param graphItem the related graph item.
	 * @param textfield the text field that display the expression.
	 * @param newValue the new value.
	 * @param oldValue the old value.
	 */
	public ChangeBinaryEdit( GraphItem graphItem, JComponent textfield, String newValue, String oldValue)  {
		_graphItem = graphItem;
		_textfield = textfield;
		_newValue = newValue;
		_oldValue = oldValue;
	}

	/**
	 * Re-applies the edit, assuming that it has been undone.
	 */
	@Override
	public void redo( ) {
		_graphItem.requestFocusInWindow();
		_graphItem.changeBinary(_textfield, _newValue, false);
	}

	/**
	 * Undoes the edit that was made.
	 */
	@Override
	public void undo( ) {
		_graphItem.requestFocusInWindow();
		_graphItem.changeBinary(_textfield, _oldValue, false);
	}
}