/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.item;

import java.awt.Color;

import fr.greyc.pantheon.ariane.graph.Graph;
import fr.greyc.pantheon.ariane.utils.TreeItem;

public abstract class ControlStructureItem extends ControlItem {
	/** The color of the item. */
	private static final Color DEFAULT_COLOR = new Color(255, 128, 228);

	/** The color of the item. */
	private static Color _color = DEFAULT_COLOR;

	public ControlStructureItem( TreeItem treeItem, Graph graph, String name ) {
		super(treeItem, graph, name);
	}
	
	public ControlStructureItem( Graph graph ) {
		super(graph);
	}

	/**
	 * Changes the current color.
	 * @param col the new color.
	 */
	public static final void setColor( Color col ) {
		_color = col;
	}

	/**
	 * Returns the current color.
	 * @return the color.
	 */
	public static final Color getColor( ) {
		return _color;
	}

	/**
	 * Returns the default color.
	 * 
	 * @return the color.
	 */
	public static final Color getDefaultColor( ) { return DEFAULT_COLOR; }

	/**
	 * Returns the background color (same as geColor()).
	 * @return the color.
	 */
	@Override
	public Color getBackgroundColor( ) {
		return _color;
	}
}
