/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.item.etc;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import fr.greyc.pantheon.ariane.graph.slot.OutputDataSlot;
import fr.greyc.pantheon.ariane.ui.PreferenceChooser;
import fr.greyc.pantheon.ariane.utils.ImageDecoder;

public final class ButtonImage extends JButton {
	private final String _text;
	private final OutputDataSlot _slot;
	private static final int MAX_HEIGHT = 80;

	public ButtonImage( OutputDataSlot slot, String text ) {
		super(text);
		setMinimumSize(new Dimension(0, MAX_HEIGHT + 8));
		setPreferredSize(new Dimension(0, MAX_HEIGHT + 8));
		_text = text;
		_slot = slot;
	}

	@Override
	public void setEnabled( boolean value ) {
		if (value) {
			new FileReader().start();
		} else {
			setIcon(null);
			setText(_text);
			setHorizontalAlignment(CENTER);
			super.setEnabled(false);
		}
	}

	private ImageIcon getThumbnail( String filename ) {
		File file = new File(filename);
		if (file.exists() && FileImagePreviewer.isText(file, false)) {
			try {
				int previewLength = 300;
				FileInputStream fin = new FileInputStream(file);
				byte[] data = new byte[previewLength];
				int bytesRead = 0;
				for (int i = 0; i < previewLength; i++) {
					int character = fin.read();
					if (character == -1) {
						break;
					}
					bytesRead++;
					data[i] = (byte) character;
				}
				String text = new String(data, 0, bytesRead);
				fin.close();

				String[] lines = text.split("\n");
				int width = 200;
				BufferedImage img = new BufferedImage(width, MAX_HEIGHT, BufferedImage.TYPE_INT_ARGB);
				Graphics2D g2d = img.createGraphics();
				g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g2d.setFont(PreferenceChooser.getFont(9));
				int yoffset = 9;
				g2d.setColor(getForeground());
				for (int i = 0; i < lines.length; i++ ) {
					g2d.drawString(lines[i], 0, yoffset * (i + 1));
				}
				ImageIcon icon = new ImageIcon(img);
				setHorizontalAlignment(LEFT);
				return icon;
			} catch (IOException ex) {
				return null;
			}
		} else { //Image
			setHorizontalAlignment(CENTER);
			return ImageDecoder.getThumbnail(filename, MAX_HEIGHT);
		}
	}

	public final class FileReader extends Thread {
		@Override
		public void run() {
			ImageIcon icon = getThumbnail(_slot.getValue());
			if (icon == null) {
				setIcon(null);
				setText(_text);
				setHorizontalAlignment(CENTER);
			} else {
				setIcon(icon);
				setText(null);
			}
			ButtonImage.super.setEnabled(true);
		}
	}
}
