/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.item.etc;

import java.io.File;

import javax.swing.Icon;
import javax.swing.filechooser.FileView;

import fr.greyc.pantheon.ariane.Ariane;

/**
 * A convenience implementation of the FileView interface that
 * manages name, icon, traversable, and file type information.
 *
 * This implementation will work well with file systems that use
 * "dot" extensions to indicate file type. For example: "picture.pan"
 * as a Pandore image.
 * 
 * @version Régis CLOUARD - Feb 01, 2001
 * @version Régis CLOUARD - Nov 27, 2011 (add new image format).
 */
public final class FileTypeViewer extends FileView {
	/**
	 * Convenience method that returns a the "dot" extension for the
	 * given file.
	 * 
	 * @param f a file.
	 * @return the extension.
	 */
	private String getExtension( File file ) {
		String name = file.getName();
		if (name != null) {
			int extensionIndex = name.lastIndexOf('.');
			if (extensionIndex < 0) {
				return null;
			}
			return name.substring(extensionIndex + 1).toLowerCase();
		}
		return null;
	}

	/**
	 * Returns the icon that represents this file.
	 * Default implementation returns null.
	 * 
	 * @param f a regular file.
	 * @return an image icon.
	 */
	@Override
	public Icon getIcon( File file ) {
		String extension = getExtension(file);
		if (extension  == null) {
			return null;
		}
		if (	   "png".equals(extension)
				|| "jpg".equals(extension)
				|| "jpeg".equals(extension)
				|| "gif".equals(extension)
				|| "bmp".equals(extension)
				|| "tif".equals(extension)
				|| "tiff".equals(extension)
				|| "ppm".equals(extension)
				|| "pbm".equals(extension)
				|| "pgm".equals(extension)
				|| "pan".equals(extension)) {
			return Ariane.getIcon("imgpan.png");
		}
		return null;
	}
}