/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.item.etc;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.EmptyBorder;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.graph.slot.InputParameterSlot;
import fr.greyc.pantheon.ariane.ui.PreferenceChooser;

/**
 * A <code>PropertyViewer</code> provides a dialog box to display graph item's
 * properties.
 * 
 * @version Régis CLOUARD -Jul 07, 2012 (Put button panel below the properties)
 */
public class PropertyViewer extends JPanel {
	/** The result value. */
	private JLabel _resultLabel;

	/** The list of all output result buttons. */
	private final List<JButton> _inputButtons = new ArrayList<JButton>();

	/** The list of all output result buttons. */
	private final List<JButton> _outputButtons = new ArrayList<JButton>();

	/** The default row height. */
	private static final int DEFAULT_ROW_HEIGHT = 25;

	/** The language used to display the properties. */
	private ResourceBundle _lang;

	/** The list of properties as a pair (name, value). */
	private final List<PropertyCouple> _properties = new ArrayList<PropertyCouple>();

	/** The list of properties as a pair (name, value). */
	private final List<PropertyCouple> _inputsoutputs = new ArrayList<PropertyCouple>();

	/**
	 * Creates an empty frame. Used for note for example.
	 */
	public PropertyViewer() {
		setOpaque(false);
	}

	/**
	 * Creates a convenient dialog box for the specified item.
	 * 
	 * @param lang the language of display.
	 * @param item the related graph item
	 */
	public PropertyViewer( ResourceBundle lang, final GraphItem item ) {
		_lang = lang;
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
				Ariane.getString("PropertyViewer.title") + " (" + item.getId() + ")"));
		
		buildPropertyList(item);

		setLayout(new BorderLayout());

		JPanel contents = new JPanel();
		contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
		Dimension dimension = new Dimension(10000, DEFAULT_ROW_HEIGHT);
		for (PropertyCouple property : _properties) {
			JPanel panel = new JPanel(new GridLayout(1, 2));
			JLabel title = new JLabel(property._title);
			title.setOpaque(true);
			title.setBorder(new EmptyBorder(0, 2, 0, 0));
			title.setFont(PreferenceChooser.getDefaultFont());
			panel.add(title);

			JComponent component = property._value;
			adjustComponentSize(component);
			panel.add(property._value);
			
			panel.setMaximumSize(dimension);
			contents.add(panel);
		}

		for (PropertyCouple property : _inputsoutputs) {
			JPanel panel = new JPanel(new BorderLayout());
			JLabel title = new JLabel(property._title);
//			title.setForeground(property._color);
			title.setOpaque(true);
			title.setBorder(new EmptyBorder(0, 2, 0, 0));
			title.setFont(PreferenceChooser.getDefaultFont());
			panel.add(title, BorderLayout.NORTH);

			JComponent component = property._value;
			adjustComponentSize(component);
			panel.add(property._value, BorderLayout.CENTER);
			
			panel.setMaximumSize(dimension);
			contents.add(panel);
		}

		JPanel buttonPanel = new JPanel(new GridLayout(1, 2));
		JButton buttonOK;
		buttonOK = new JButton(Ariane.getString("Graph.Menu.execute"));
		buttonOK.setFont(PreferenceChooser.getDefaultFont());
		buttonOK.addActionListener(new ActionListener() {
			@Override
			public final void actionPerformed( ActionEvent event ) {
				item.getGraph().execute();
			}
		});
		buttonPanel.add(buttonOK);

		JButton buttonReset = new JButton(Ariane.getString("Graph.Menu.reset"));
		buttonReset.setFont(PreferenceChooser.getDefaultFont());
		buttonReset.addActionListener(new ActionListener() {
			@Override
			public final void actionPerformed( ActionEvent event ) {
				item.resetAll();
			}
		});
		dimension = buttonPanel.getPreferredSize();
		dimension.width = 10000;
		buttonPanel.setMaximumSize(dimension);
		buttonPanel.add(buttonReset);
		contents.add(buttonPanel);
		contents.add(Box.createVerticalGlue());

		JScrollPane scrollPane = new JScrollPane(contents,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		add(scrollPane, BorderLayout.CENTER);
	}

	private void adjustComponentSize( final JComponent component ) {
		int height = component.getPreferredSize().height;
		Dimension dimension = new Dimension(0, height);
		component.setPreferredSize(dimension);
	}

	/**
	 * @return the current language used to display the properties.
	 */
	public final ResourceBundle getLanguage() {
		return _lang;
	}

	/**
	 * This class is a simple holder for a property pair (a title and an editor
	 * for the value).
	 */
	public static final class PropertyCouple {
		/** The title of the property. */
		private final String _title;
		/** The value of the property. */
		private final JComponent _value;

		/**
		 * Create a property pair title value.
		 */
		public PropertyCouple( String title, JComponent value ) {
			_title = title;
			value.setFont(PreferenceChooser.getDefaultFont());
			_value = value;
		}
	}

	/**
	 * Builds the list of properties as an array list. Template Method Design
	 * Pattern
	 * @param item the related graph item.
	 */
	private void buildPropertyList( final GraphItem item ) {
		{
			JComponent component = item.getNameEditor();
			_properties.add(new PropertyCouple(Ariane.getString("PropertyViewer.name"), component));
		}
		if (item.getCategory() != null) {
			JTextField component = new JTextField(item.getCategory());
			component.setEnabled(false);
			_properties.add(new PropertyCouple(Ariane.getString("PropertyViewer.category"), component));
		}

		JComponent editor = item.getBinaryEditor();
		if (editor != null) {
			_properties.add(new PropertyCouple(Ariane.getString("PropertyViewer.binary"), editor));
		}

		{
			JButton jbuttonman;
			jbuttonman = new JButton(Ariane.getString("PropertyViewer.manual"));
			jbuttonman.setFont(PreferenceChooser.getDefaultFont());
			jbuttonman.addActionListener(new ActionListener() {
				@Override
				public final void actionPerformed( ActionEvent event ) {
					item.showMan();
				}
			});
			_properties.add(new PropertyCouple(Ariane.getString("PropertyViewer.documentation"), jbuttonman));
		}
		if (item.getFormula() != null && item.displayFormula()) {
			JComponent component = item.getFormulaEditor();
			_properties.add(new PropertyCouple(item.getFormulaLabel(), component));
		}
		List<PropertyCouple> additionalProperties = item.getSpecificProperties();
		if (additionalProperties != null) {
			_properties.addAll(additionalProperties);
		}
		// Parameters.
		for (int i = 0; i < item.getInputParameterSlots().size(); i++) {
			InputParameterSlot isp = item.getInputParameterSlots().get(i);
			JComponent component = isp.getParameterEditor();
			_properties.add(new PropertyCouple(item.getInputParameterSlots().get(i).getName(), component));
		}
		// Input images.
		for (int i = 0; i < item.getInputDataSlots().size(); i++) {
			JButton button = item.getInputDataSlots().get(i).getButton();
			if (item.getExecutionStatus() == GraphItem.Status.EXECUTED
					&& item.getInputDataSlots().get(i).isValidate()) {
				button.setEnabled(true);
			} else {
				button.setEnabled(false);
			}
			_inputButtons.add(button);
			_inputsoutputs.add(new PropertyCouple(Ariane.getString("PropertyViewer.input") + " #" + i, button));
		}
		// Output images.
		for (int i = 0; i < item.getOutputDataSlots().size(); i++) {
			JButton button = item.getOutputDataSlots().get(i).getButton();
			if (item.getExecutionStatus() == GraphItem.Status.EXECUTED && item.getOutputDataSlots().get(i).isValidate()) {
				button.setEnabled(true);
			} else {
				button.setEnabled(false);
			}
			_outputButtons.add(button);
			_inputsoutputs.add(new PropertyCouple(Ariane.getString("PropertyViewer.output")
					+ " #"
					+ i
					+ " (ctrl-"
					+ i + ")", button));
		}
		// Result
		if (item.getOutputResultSlots().size() > 0) {
			if (item.getExecutionStatus() == GraphItem.Status.EXECUTED) {
				_resultLabel = new JLabel(item.getOutputResultSlot().getValue());
			} else {
				_resultLabel = new JLabel(Ariane.getString("PropertyViewer.result.unbound"));
			}
			_properties.add(new PropertyCouple(Ariane.getString("PropertyViewer.result"), _resultLabel));
		}
	}

	/**
	 * Updates the contents of the panel.
	 * 
	 * @param item the related item.
	 */
	public final void update( GraphItem item ) {
		boolean updated = false;
		if (_resultLabel != null) {
			updated = true;
			if (item.getExecutionStatus() == GraphItem.Status.EXECUTED) {
				_resultLabel.setText(item.getOutputResultSlot().getValue());
			} else {
				_resultLabel.setText(Ariane.getString("PropertyViewer.result.unbound"));
			}
		}
		if (!_inputButtons.isEmpty()) {
			for (int i = 0; i < item.getInputDataSlots().size(); i++) {
				updated = true;
				if (item.getExecutionStatus() == GraphItem.Status.EXECUTED
						&& item.getInputDataSlots().get(i).isValidate()) {
					_inputButtons.get(i).setEnabled(true);
				} else {
					_inputButtons.get(i).setEnabled(false);
				}
			}
		}
		if (!_outputButtons.isEmpty()) {
			for (int i = 0; i < item.getOutputDataSlots().size(); i++) {
				updated = true;
				if (item.getExecutionStatus() == GraphItem.Status.EXECUTED && item.getOutputDataSlots().get(i).isValidate()) {
					_outputButtons.get(i).setEnabled(true);
				} else {
					_outputButtons.get(i).setEnabled(false);
				}
			}
		}
		if (updated) {
			revalidate();
			repaint();
		}
	}
}