/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.utils.GDialog;
import fr.greyc.pantheon.ariane.utils.Tree;
import fr.greyc.pantheon.ariane.xml.XMLElement;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * The class <code>LibraryPathSetting</code> defines a dialog box
 * where the user can set the operator library path.
 * 
 * @author Frédéric LAIR - Jul 03, 2007
 * @version Régis CLOUARD - Oct 07, 2007
 */
public final class LibraryPathSetting extends GDialog {
	private String _operatorLibraryPath;

	/**
	 * Creates a dialog box for setting new paths.
	 */
	public LibraryPathSetting( ) {
		super((JFrame)null, Ariane.getString("LibraryPathSetting.title"), true);
		setResizable(false);
		displayContent();
		setLocationRelativeTo(null);
		addWindowListener(new WindowAdapter() {
			// Redirects the window closing event to cancelAction();
			@Override
			public void windowClosing( WindowEvent event ) {
				cancelAction();
			}
		});
	}

	/**
	 * Initializes the content of the dialog box.
	 */
	private void initializeContent( ) {
		_operatorLibraryPath = Ariane.getLibraryPath();
	}

	/**
	 * Changes the configuration with the user's choice.
	 */
	private void changeContent( ) {
		if (!PreferenceChooser.getBoolean(PreferenceChooser.USE_OPERATOR_LIBRARY)) {
			return;
		}
		String libraryFilename = _operatorLibraryPath + File.separator + "lib" + File.separator + Ariane.LIBRARY_CONFIG_FILENAME;
		try {
			List<XMLElement> configHandler = Ariane.readXMLFile(new File(libraryFilename));
			Tree operatorTree = Ariane.getWorkspace().getTree();
			operatorTree.setCommonOperators(configHandler);
			Ariane.setLibraryPath(_operatorLibraryPath);
		} catch (IOException e) {
			Ariane.getWorkspace().printErrorMessage(Ariane.getString("Workspace.addsubroutine.error") + " " + libraryFilename + ". " + e.getLocalizedMessage());
		}
	}

	/**
	 * Displays the content of the dialog box.
	 */
	private void displayContent( ) {
		initializeContent();

		final JTextField textfield2 = new JTextField(30);
		textfield2.setText(_operatorLibraryPath);
		textfield2.setCaretPosition(0);
		textfield2.moveCaretPosition(_operatorLibraryPath.length());
		textfield2.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_operatorLibraryPath = ((JTextField)event.getSource()).getText();
			}
		});

		JButton browser = new JButton(Ariane.getString("Ariane.browse"));
		browser.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				final JFileChooser fc2 = new JFileChooser();
				fc2.setCurrentDirectory(new File(Ariane.getLibraryPath()));
				fc2.setDialogType(JFileChooser.OPEN_DIALOG);
				fc2.setDialogTitle(Ariane.getString("FileChooser.titleSelectDir"));
				fc2.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int returnVal = fc2.showOpenDialog(LibraryPathSetting.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					_operatorLibraryPath = fc2.getSelectedFile().getAbsolutePath();
					textfield2.setText(_operatorLibraryPath);
					File file = new File(_operatorLibraryPath + File.separator + "lib" + File.separator + Ariane.LIBRARY_CONFIG_FILENAME);
					if (!file.exists()) {
						JOptionPane.showMessageDialog(LibraryPathSetting.this, Ariane.getString("LibraryPathSetting.alertMessage"),
								Ariane.getString("LibraryPathSetting.alertTitle"), JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});

		// Apply button
		JButton applyButton = new JButton(Ariane.getString("LibraryPathSetting.apply"));
		applyButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				applyAction();
			}
		} );

		// Cancel button
		JButton cancelButton = new JButton();
		Action cancelKeyAction = new AbstractAction(Ariane.getString("LibraryPathSetting.cancel")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				cancelAction();
			}
		};
		cancelButton.setAction(cancelKeyAction);

		JPanel panel1 = new JPanel();
		panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("LibraryPathSetting.libraryPathName")));

		JLabel label = new JLabel(Ariane.getString("LibraryPathSetting.libraryPathDescription"));
		JPanel panel2 = new JPanel(new GridLayout(3, 1));
		panel2.add(label);

		JPanel panel3 = new JPanel();
		panel3.add(textfield2);
		panel3.add(browser);

		panel2.add(panel3);
		final JCheckBox checkBox = new JCheckBox(Ariane.getString("LibraryPathSetting.useOperatorLibrary"), !PreferenceChooser.getBoolean(PreferenceChooser.USE_OPERATOR_LIBRARY));
		checkBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				PreferenceChooser.putBoolean(PreferenceChooser.USE_OPERATOR_LIBRARY, !((JCheckBox)event.getSource()).isSelected());
			}
		});
		panel2.add(checkBox);
		panel1.add(panel2);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(applyButton);
		buttonPanel.add(cancelButton);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(panel1, BorderLayout.CENTER);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		pack();
		setDefaultButton(applyButton);
		setCancelButton(cancelButton);
		textfield2.requestFocusInWindow();
	}

	/**
	 * Closes the preferences dialog box.
	 */
	private void cancelAction( ) {
		setVisible(false);
		dispose();
	}

	/**
	 * Applies and saves the chosen preferences,
	 * and closes the dialog box.
	 */
	private void applyAction( ) {
		changeContent();
		if (PreferenceChooser.save()) {
			Ariane.getWorkspace().repaintAllGraphs();
		}
		setVisible(false);
		dispose();
	}
}