/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.utils.GDialog;
import fr.greyc.pantheon.ariane.utils.Tree;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * The class <code>SearchDialog</code> opens a dialog
 * that allows the user to search operators in the tree
 * by name.
 * 
 * @author Régis CLOUARD - Dec 04, 2007
 */
public final class SearchDialog extends GDialog {
	/** The workspace. */
	private final Tree _tree;
	/** The name of the operator to be search. */
	private JTextField _textfield2;

	/**
	 * Creates a dialog box for searching operators.
	 * 
	 * @param tree the tree that holds the operators.
	 */
	public SearchDialog( Tree tree ) {
		super((JFrame)null, Ariane.getString("SearchDialog.title"), true);
		_tree = tree;
		setResizable(false);
		displayContent();
		setLocationRelativeTo(Ariane.getFrame());
		setAlwaysOnTop(true);
		addWindowListener(new WindowAdapter() {
			// Redirects the window closing event to cancelAction();
			@Override
			public void windowClosing( WindowEvent event ) {
				cancelAction();
			}
		});
		_textfield2.requestFocusInWindow();
	}

	/**
	 * Displays the content of the dialog box.
	 */
	private void displayContent( ) {
		Box panel = Box.createVerticalBox();

		JPanel panel1 = new JPanel();
		panel.add(panel1);
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		Box panel2 = Box.createVerticalBox();
		panel1.add(panel2);

		_textfield2 = new JTextField(30);
		JLabel label = new JLabel(Ariane.getString("SearchDialog.text"));
		panel2.add(label);
		panel2.add(_textfield2);
		JPanel buttonPanel = new JPanel();

		JButton applyButton = new JButton(Ariane.getString("SearchDialog.ok"));
		applyButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				applyAction();
			}
		});

		JButton cancelButton = new JButton();
		cancelButton.setAction(new AbstractAction(Ariane.getString("SearchDialog.cancel")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				cancelAction();
			}
		});
		buttonPanel.add(applyButton);
		buttonPanel.add(cancelButton);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(panel1, BorderLayout.CENTER);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		setDefaultButton(applyButton);
		setCancelButton(cancelButton);
		pack();
	}

	/**
	 * Closes the preferences dialog box.
	 */
	private void cancelAction( ) {
		setVisible(false);
		dispose();
	}

	/**
	 * Applies and saves the chosen preferences,
	 * and closes the dialog box.
	 */
	private void applyAction( ) {
		String pattern = _textfield2.getText();
		if (!pattern.isEmpty()) {
			_tree.search(pattern);
		}
		setVisible(false);
		dispose();
	}
}
